using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

using VirtualLab.Programming;
using VirtualLab.Design;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Functions;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Numerics.Region2D;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.Propagation;
using VirtualLabAPI.Core.SupportFunctions;

namespace OwnCode {
    public class VLModule {
        /// <summary>
        /// Imports point cloud data (a list of x-y-z coordinates) from a text file into a Gridless Data Array.
        /// Each line should contain three values (x, y, and z) separated by a space.
        /// Author: Sahil Rajan &amp; Torsten Schöning (both Wyrowski Photonics GmbH)
        /// Version: 1.2, released under CC-BY 4.0 license on 2020-03-31
        /// </summary>
        public void Run() {
            // begin of user input
            string fileName = @"D:\surface profile.xyz";
            // sampling distance in x and y direction in meters
            VectorD samplingDistanceOfEquidistantData = new VectorD(10e-5, 10e-5);
            // scaling of height data in meters
            double Z_Scaling_factor = 1e-3;
            // end of user input      
          
            // count lines by for loop
            string[] text = File.ReadAllLines(fileName);            
            List<string> newText = new List<string>(text.Length);
            
            // Remove Comment Lines from Text File
            int CommentLines = 0;
            bool isCommentLine = true;
            for(int i = 0; i < text.Length; i++) {
                if(isCommentLine){                    
                    CommentLines++;
                    if(text[i].Contains('#')){
                        isCommentLine = false;
                    }
                }else{                    
                    if(text[i].Contains('#')){
                        break;
                    }
                    newText.Add(text[i]);
                }
            }           
            
            int NumberofLines = newText.Count;
            
            ComplexField coordinates = new ComplexField(new Vector(2, NumberofLines));            
            int numberDataPointsXY = (int) (Math.Sqrt(NumberofLines));
            
            ComplexField cfEquidistantData = new ComplexField(new Vector(numberDataPointsXY, numberDataPointsXY), false, PrecisionMode.Double);
            cfEquidistantData.Fill(0);
            
            for (int lineIndex = 0; lineIndex < NumberofLines; lineIndex++) {
                string[] fields = newText[lineIndex].Split(' ');
                try {   double Coordinates_X = (Convert.ToDouble(fields[0]));
                    double Coordinates_Y = (Convert.ToDouble(fields[1]));

                    if (fields[2].Contains('N')) {

                    }
                    else {
                        double Data_Z = (Convert.ToDouble(fields[2])) * Z_Scaling_factor;
                        
                        //store data in equidistant value matrix
                        cfEquidistantData[(int) Coordinates_X, (int) Coordinates_Y] = Data_Z;
                    }
                }
                catch {
                    throw new ArgumentException("Line #" + (lineIndex + 1).ToString("n0") + " could not be parsed.");
                }
            }
            
            SamplingParameters sPara = new SamplingParameters();
            sPara.SamplingDistance = samplingDistanceOfEquidistantData;
            sPara.SamplingPoints = cfEquidistantData.SamplingPoints;
            
            DataArray2D daGriddedData = new DataArray2D(new ComplexFieldArray(new ComplexField[]{cfEquidistantData}),
                new PhysicalProperty[]{PhysicalProperty.Length},
                new string []{"Height Values"},
                sPara.SamplingDistance.X,
                (-0.5 * sPara.Diameter.X) + (0.5 * sPara.SamplingDistance.X),
                PhysicalProperty.Length,
                "X",
                sPara.SamplingDistance.Y,
                (-0.5 * sPara.Diameter.Y) + (0.5 * sPara.SamplingDistance.Y),
                PhysicalProperty.Length,
                "Y");
            
            Globals.DataDisplayInternal.ShowDocument(daGriddedData, "Result of Import (Equidistant)", duplicateDocument: true);     
            
        }
    }
}